#!/usr/bin/env python
import subprocess
import os
import unittest
import cStringIO as StringIO
import tempfile
import shutil
import json


import yaml

SRC_PATH = os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        '..',
        'src')
)

PROJECT_BASE_PATH = os.path.abspath(
    os.path.dirname(__file__))

os.environ["PATH"] = SRC_PATH + os.pathsep + os.environ["PATH"]


def run(working_path, *args):
    with tempfile.TemporaryFile() as f:
        os.chdir(working_path)
        ret = subprocess.call(args, stdout=f)
        f.flush()
        f.seek(0)
        return ret, "%s" % f.read()


def run_project(prj_name, *args):
    prj_path = os.path.join(PROJECT_BASE_PATH, prj_name)
    return run(prj_path, *args)

DEFAULT_PROD_YAML = '''
name: prod
number: 1
updated: '2015-01-01 00:00:00'
'''

CMD="main.py"

class Prj1Test(unittest.TestCase):
    def setUp(self):
        self.prj_name = 'prj1'
        self.prj_path = os.path.join(
            PROJECT_BASE_PATH, self.prj_name)

        with open(os.path.join(self.prj_path, 'prod.yaml'), 'wb') as f:
            f.write(DEFAULT_PROD_YAML)

        self.build_path = os.path.join(self.prj_path, '_builds')
        shutil.rmtree(self.build_path, True)

    def test_0new(self):
        ret, conf_id = run_project(
            self.prj_name,
            CMD, 'id', '--conf', 'prod.yaml')
        self.assertEqual(conf_id.strip(), 'prod/1')

        ret, stdout = run_project(
            self.prj_name,
            CMD, 'next', '--conf', 'prod.yaml')

        self.assertEqual(ret, 0)

        self.assertEqual(
            stdout,
            "Conf number increased to [2]\nupdating file: prod.yaml\n")

        ret, conf_id = run_project(
            self.prj_name,
            CMD, 'id', '--conf', 'prod.yaml')
        self.assertEqual(conf_id.strip(), 'prod/2')

    def test_build(self):
        ret, stdout = run_project(
            self.prj_name,
            CMD, 'build', '--conf', 'prod.yaml')
        self.assertEqual(ret, 0)
        self.assertEqual(
            stdout, """\
with [conf=prod.yaml] 
build started prod/1 ... [{prj_path}/_builds/prod/1]
building: m1 (module)
b_can_copy copied from ({prj_path}/m1/b_can_copy)
run.sh <== [run.sh.tpl]

build finished\n""".format(prj_path=self.prj_path))

        m1_path = os.path.join(self.prj_path, '_builds', 'prod', '1', 'm1', 'module.yaml')

        with open(m1_path) as f:
            m1 = yaml.load(f)
            self.assertEqual(m1, {
                '_ignores': ['*.not_copy', 'should_not_be_copy'],
                'name': 'awesome_project-prod-1'
            })

    def test_folder_structure(self):
        '''
        testing the folder structure, also for module _ignores
        '''
        ret, stdout = run_project(
            self.prj_name,
            CMD, 'build', '--conf', 'prod.yaml')

        flist = list(os.walk(
            os.path.join(self.prj_path, '_builds', 'prod')))
        flist_should = [
            (
                os.path.join(self.prj_path, '_builds', 'prod'),
                ['1'],
                []
            ),

            (
                os.path.join(self.prj_path, '_builds', 'prod', '1'),
                ['m1'],
                []
            ),

            (
                os.path.join(self.prj_path, '_builds', 'prod', '1', 'm1'),
                [],
                ['b_can_copy', 'module.yaml', 'run.sh']
            ),

        ]

        self.assertEqual(
            flist_should, flist)

        self.assertEqual(stdout, """\
with [conf=prod.yaml] 
build started prod/1 ... [{prj_path}/_builds/prod/1]
building: m1 (module)
b_can_copy copied from ({prj_path}/m1/b_can_copy)
run.sh <== [run.sh.tpl]

build finished\n""".format(prj_path=self.prj_path))

    @unittest.skip("fix it later.")
    def test_inspect(self):
        '''
        testing the folder structure, also for module _ignores
        '''
        ret, stdout = run_project(
            self.prj_name,
            CMD, 'inspect', '--conf', 'prod.yaml')
        cnt = '\n'.join(stdout.split('\n')[1:])
        d = json.loads(cnt)
        self.assertEqual(
            d, {
                "project": {
                "decription": "awesome_decription",
                "_executable": [
                "*.sh",
                "*.py"
            ],
            "name": "awesome_project",
            "_varient": "prod uat"
          },
          "m1": {
            "_ignores": [
              "*.not_copy",
              "should_not_be_copy"
            ],
            "name": "awesome_project-prod-1"
          },
          "conf": {
            "updated": "2015-01-01 00:00:00",
            "name": "prod",
            "number": 1
          }
        })

    def test_create(self):
        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import argparse
import re
import os
import os.path
from collections import namedtuple
import json
import datetime
import sys
import shutil
import fnmatch

import yaml
import jinja2

ENV_ARGS_CFG = 'AGC_CONF'
ENV_ARGS_BOIL = 'AGC_BOIL'


def load_version():
    path = os.path.join(
        os.path.dirname(
            os.path.realpath(__file__)),
        'VERSION.txt')
    with open(path) as f:
        return f.read().strip()

VERSION = load_version()


def aws_userdata(arg_lst):
    lines = []
    path = os.path.join(*arg_lst)
    with open(path) as f:
        for l in f:
            lines.append("%s\n" % l.strip())

    return json.dumps({
        'Fn::Base64': {
            'Fn::Join': [
                '',
                lines
            ]
        }
    }, indent=4)


def template_env(*tpl_folder_lst):
    env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(tpl_folder_lst)
    )
    env.filters['jsonify'] = json.dumps
    env.filters['yamlify'] = yaml.dump
    env.filters['aws_userdata'] = aws_userdata
    return env


class Project(object):
    def __init__(self, proj_folder, build_conf):
        self.proj_folder = proj_folder
        self.modules = []
        self.list_modules()
        self.the_config = {'conf': build_conf}

        self.load_config()

    def list_modules(self):
        '''has to run before config'''
        (fullpath, folders, files) = next(
            os.walk(self.proj_folder))
        self.modules = [
            f for f in folders if f[0] not in ['_', '.']
        ]

    def load_config(self):
        self.load_config_from_project()

        for m in self.modules:
            self.load_config_by_module(m)

    def load_config_from_project(self):
        tpl_env = template_env(self.proj_folder)
        tpl = tpl_env.get_template('project.yaml')

        txt_conf = tpl.render(conf=self.the_config['conf'])
        self.the_config['project'] = yaml.load(txt_conf)

    def load_config_by_module(self, m):
        tpl_env = template_env(
            os.path.join(self.proj_folder, m))
        tpl = tpl_env.get_template('module.yaml')
        txt_conf = tpl.render(**self.the_config)
        self.the_config[m] = yaml.load(txt_conf)

    def build(self):
        for m in self.modules:
            dst_folder = os.path.join(
                self.dst_base_folder(), m)
            self._build(True, m, dst_folder)

        if os.path.isdir(os.path.join(self.proj_folder, '_script')):
            self._build(False, '_script', self.dst_base_folder())

    def build_id(self, seperator='/'):
        return seperator.join([
            str(self.the_config['conf']['name']),
            str(self.the_config['conf']['number'])
        ])

    def dst_base_folder(self):
        return os.path.join(
            self.proj_folder,
            '_builds',
            str(self.the_config['conf']['name']),
            str(self.the_config['conf']['number'])
        )

    def dst_base_tmp_folder(self):
        return os.path.join(
            self.proj_folder,
            '_builds',
            '.tmp',
            str(self.the_config['conf']['name']),
            str(self.the_config['conf']['number'])
        )

    def _build(self, is_module, m, dst_folder):
        src_folder = os.path.join(self.proj_folder, m)
        if is_module:
            print "building: %s (module)" % m
        else:
            print "building: %s" % m


        if not os.path.exists(dst_folder):
            os.makedirs(dst_folder)

        if is_module:
            # 1. produce the module.yaml
            dst_module_cfg = os.path.join(dst_folder, 'module.yaml')
            with open(dst_module_cfg, 'wb') as f:
                yaml.dump(self.the_config[m], f, default_flow_style=False)

        tpl_env = template_env(self.proj_folder, src_folder)
        fullpath, folders, files = next(os.walk(src_folder))

        params = dict(self.the_config)
        params['_BUILD_DST_FOLDER'] = dst_folder

        if is_module:
            params.update(self.the_config[m])

        files = sorted(files)
        files_tpl = [
            fname for fname in files if fname.endswith('.tpl')]
        files_cp = [
            fname for fname in files if fname not in files_tpl]

        if is_module:
            _ignores = self.the_config[m].get('_ignores', [])

            for p in _ignores:
                files_cp = [
                    f for f in files_cp if not fnmatch.fnmatch(f, p)]


        for fname in files_cp:
            if fname not in ('module.yaml', ):
                src_path = os.path.join(src_folder, fname)
                dst_path = os.path.join(dst_folder, fname)
                shutil.copyfile(src_path, dst_path)
                print "%s copied from (%s)" % (fname, src_path)

        for fname in files_tpl:
            path = os.path.join(src_folder, fname)
            dst_path = os.path.join(dst_folder, fname[:-4])
            # read the template
            with open(path) as f:
                tpl = tpl_env.get_template(fname)
                cnt = tpl.render(params)
                # render it to dst folder
                with open(dst_path, 'wb') as f2:
                    f2.write(cnt)
                    print "%s <== [%s]" % (
                        dst_path[len(dst_folder)+1:], fname)

        # empty line make it easier to see
        print "" 


def get_conf(args_confpath):
    conf_path = os.environ.get(ENV_ARGS_CFG) if not args_confpath else args_confpath

    if not conf_path:
        sys.exit("""No config is provided. Please specify it in 
enviornment variable %s, or as option argument --conf.""" % ENV_ARGS_CFG)

    if not os.path.isfile(conf_path):
        sys.exit("The config file %s is invalid." % conf_path)

    try:
        with open(conf_path) as f:
            return conf_path, yaml.load(f)
    except:
        sys.exit("failed to open build conf file: %s" % conf_path)


def retire():
    pass




#!/usr/bin/env python
from setuptools import setup, find_packages
import json
import os


def read(fname):
    with open(fname) as f:
        return f.read()


def load_version():
    with open('src/agile_conf/VERSION.txt') as f:
        return f.read().strip()


setup(
    name='agile_conf',
    version=load_version(),
    description='''A config files (in [YAML](http://yaml.org) format) and \
template engine ([Jinja2](http://jinja.pocoo.org)) \
based configuration compile / management tool to make DevOp tasks''',
    long_description=read('README.txt'),
    license='BSD',
    author='Tom Tang',
    author_email='tly1980@gmail.com',
    url="https://github.com/tly1980/agile_conf",
    platforms='any',
    install_requires=[
        'PyYAML>=3.0',
        'Jinja2',
        'docopt>=0.6.2'
    ],
    scripts=['scripts/agc'],
    package_dir={'': 'src'},
    packages=['agile_conf'],
    package_data={'agile_conf': ['*.txt']},
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Code Generators'
    ],
)
